@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -47,8 +47,8 @@ class AwlInsn_MU_I(AwlInsn): #+cdef
 		prod *= self.cpu.accu1.getSignedWord()
 		self.cpu.accu1.setDWord(prod)
 		if self.cpu.is4accu:
-			self.cpu.accu2.setDWord(self.cpu.accu3.getDWord())
-			self.cpu.accu3.setDWord(self.cpu.accu4.getDWord())
+			self.cpu.accu2.copyFrom(self.cpu.accu3)
+			self.cpu.accu3.copyFrom(self.cpu.accu4)
 		if prod == 0:
 			s.A1, s.A0, s.OV = 0, 0, 0
 		elif prod < 0:
