@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,5 +56,7 @@ class AwlInsn_R(AwlInsn): #+cdef
 			s.OR, s.NER = 0, 0
 		else:
 			if s.VKE and (not cpu.mcrActive or cpu.mcrIsOn()):
-				cpu.store(oper, 0, self._widths_1)
+				cpu.store(oper,
+					  constMemObj_1bit_0,
+					  self._widths_1)
 			s.OR, s.STA, s.NER = 0, s.VKE, 0
