@@ -26,12 +26,12 @@ from awlsim.common.compat import *
 from awlsim.common.cpuconfig import *
 from awlsim.common.datatypehelpers import * #+cimport
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.instructions.types import * #+cimport
 from awlsim.core.instructions.parentinfo import *
 from awlsim.core.operatortypes import * #+cimport
 from awlsim.core.operators import * #+cimport
-from awlsim.core.util import *
 
 from awlsim.awlcompiler.optrans import *
 
@@ -286,7 +286,7 @@ class AwlInsn(object): #+cdef
 		"""Check whether we have the required operator count.
 		counts is a list/set/int of possible counts.
 		"""
-		assert(self.opCount == len(self.ops))
+		assert(self.opCount == len_u32(self.ops))
 		counts = toList(counts)
 		if self.opCount not in counts:
 			raise AwlSimError("Invalid number of operators. "
@@ -344,11 +344,14 @@ class AwlInsn(object): #+cdef
 			return -1
 		return self.parentInfo.rawInsn.getLineNr()
 
+	def getParentInfo(self):
+		return self.parentInfo
+
 	def run(self): #+cdef
 		"""Run the instruction.
 		The default implementation does nothing.
 		"""
-		pass
+		pass #@nocov
 
 	def _warnDeprecated(self, moreText=""):
 		lineNrStr = ""
@@ -360,7 +363,11 @@ class AwlInsn(object): #+cdef
 		printWarning("Found DEPRECATED instruction%s:\n  %s%s" % (
 			     lineNrStr, str(self), moreText))
 
-	def getStr(self, compact=True, withSemicolon=False, withComment=False):
+	def getStr(self,
+		   compact=True,
+		   withSemicolon=False,
+		   withComment=False,
+		   withParentInfo=True):
 		ret = []
 		if self.hasLabel():
 			labelStr = self.getLabel() + ":"
@@ -377,7 +384,7 @@ class AwlInsn(object): #+cdef
 			type2name = AwlInsn.type2name_german
 		try:
 			name = type2name[self.insnType]
-		except KeyError:
+		except KeyError: #@nocov
 			name = "<unknown type %d>" % self.insnType
 		ret.append(name)
 		if self.ops:
@@ -393,14 +400,15 @@ class AwlInsn(object): #+cdef
 		if withSemicolon:
 			ret.append(";")
 		text = "".join(ret)
-		if withComment and (self.commentStr or self.parentInfo):
+		withParentInfo = withParentInfo and self.parentInfo
+		if withComment and (self.commentStr or withParentInfo):
 			text += " " * (40 - len(text))
 			text += "// "
 			if self.commentStr:
 				text += self.commentStr
-				if self.parentInfo:
-					text += "    "
-			if self.parentInfo:
+				if withParentInfo:
+					text += " "
+			if withParentInfo:
 				text += str(self.parentInfo)
 		return text
 
