@@ -2,7 +2,7 @@
 #
 # AWL simulator - labels
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,13 +25,13 @@ from awlsim.common.compat import *
 
 from awlsim.common.sources import AwlSource
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.operatortypes import * #+cimport
 from awlsim.core.operators import * #+cimport
-from awlsim.core.util import *
 
 
-class AwlLabel(object):
+class AwlLabel(object): #+cdef
 	"""AWL label."""
 
 	__slots__ = (
@@ -82,3 +82,18 @@ class AwlLabel(object):
 			if lbl.getLabelName() == label:
 				return i
 		return None
+
+	@classmethod
+	def generateLabelName(cls, index):
+		"""Generate a label name string.
+		The name does not include the final ':' character.
+		"""
+		maxIndex = (26 ** 4) - 1
+		if index < 0 or index > maxIndex:
+			raise ValueError("Label index %d out of range 0 - %d." % (
+					 index, maxIndex))
+		labelChars = [None] * 4
+		for i in range(3, -1, -1):
+			labelChars[i] = chr(ord("A") + (index % 26))
+			index //= 26
+		return "".join(labelChars)
