@@ -31,6 +31,7 @@ from awlsim.core.datatypes import *
 from awlsim.core.memory import * #+cimport
 from awlsim.core.offset import * #+cimport
 
+#from libc.stdlib cimport abort #@cy
 #from cpython.mem cimport PyMem_Malloc, PyMem_Free #@cy
 
 
@@ -59,7 +60,7 @@ class LStackAllocator(object): #+cdef
 #@cy	def __init__(self, uint32_t maxSize):
 		# maxSize -> max size of the L-stack, in bytes.
 
-		self.topFrame = None #@cy-NoneToNULL
+		self.topFrame = None #+NoneToNULL
 		self.resize(maxSize)
 
 	# Reset all allocations on the L-stack.
@@ -70,7 +71,8 @@ class LStackAllocator(object): #+cdef
 		self.memory = AwlMemory(maxAllocBytes)
 		self.maxAllocBits = maxAllocBytes * 8
 
-	def reset(self): #+cdef
+	def reset(self): #@nocy
+#@cy	cdef void reset(self):
 #@cy		cdef LStackFrame *frame
 #@cy		cdef LStackFrame *oldFrame
 
@@ -81,17 +83,19 @@ class LStackAllocator(object): #+cdef
 			oldFrame = frame
 			self.topFrame = frame = frame.prevFrame
 
-			oldFrame.prevFrame = None		#@cy-NoneToNULL
+			oldFrame.prevFrame = None		#+NoneToNULL
 			self.__frameFreeSet.add(oldFrame)	#@nocy
 #@cy			PyMem_Free(oldFrame)
 
-	def enterStackFrame(self): #+cdef
+	def enterStackFrame(self): #@nocy
+#@cy	cdef void enterStackFrame(self):
 #@cy		cdef LStackFrame *frame
 #@cy		cdef uint32_t globAllocBits
+#@cy		cdef uint32_t prevAllocBits
 
 		prevAllocBits = self.globAllocBits
-		self.globAllocBits = globAllocBits = (((prevAllocBits + 7) >> 3) << 3)
-		globAllocBytes = globAllocBits // 8
+		self.globAllocBits = globAllocBits = (((prevAllocBits + 7) >> 3) << 3)	#+suffix-u
+		globAllocBytes = globAllocBits // 8					#+suffix-u
 
 		# Allocate a new stack frame.
 		try:						#@nocy
@@ -100,7 +104,8 @@ class LStackAllocator(object): #+cdef
 			frame = LStackFrame()			#@nocy
 #@cy		frame = <LStackFrame *>PyMem_Malloc(sizeof(LStackFrame))
 #@cy		if frame == NULL:
-#@cy			raise AwlSimError("enterStackFrame: Out of memory")
+#@cy			printError("enterStackFrame: Out of memory")
+#@cy			abort()
 
 		frame.byteOffset = globAllocBytes
 		# Account the rounded-up bits to the new frame.
@@ -110,9 +115,11 @@ class LStackAllocator(object): #+cdef
 		self.topFrame = frame
 		self.topFrameOffset = make_AwlOffset(globAllocBytes, 0)
 
-	def exitStackFrame(self): #+cdef
+	def exitStackFrame(self): #@nocy
+#@cy	cdef void exitStackFrame(self):
 #@cy		cdef LStackFrame *frame
 #@cy		cdef LStackFrame *topFrame
+#@cy		cdef uint32_t globAllocBits
 
 		frame = self.topFrame
 		topFrame = self.topFrame = frame.prevFrame
@@ -124,7 +131,7 @@ class LStackAllocator(object): #+cdef
 		else:
 			self.topFrameOffset = make_AwlOffset(0, 0)
 
-		frame.prevFrame = None		#@cy-NoneToNULL
+		frame.prevFrame = None		#+NoneToNULL
 		self.__frameFreeSet.add(frame)	#@nocy
 #@cy		PyMem_Free(frame)
 
@@ -145,19 +152,19 @@ class LStackAllocator(object): #+cdef
 		roundBits = 0
 		if nrBits == 1:
 			# Bit-aligned allocation
-			offset = make_AwlOffset(globAllocBits // 8 - frame.byteOffset,
-						globAllocBits % 8)
+			offset = make_AwlOffset(globAllocBits // 8 - frame.byteOffset,	#+suffix-u
+						globAllocBits % 8)			#+suffix-u
 		else:
 			# Byte-aligned allocation
-			if globAllocBits & 7:
+			if globAllocBits & 7:						#+suffix-u
 				# Round up to the next byte boundary.
-				roundBits = 8 - (globAllocBits & 7)
+				roundBits = 8 - (globAllocBits & 7)			#+suffix-u
 				globAllocBits += roundBits
-			offset = make_AwlOffset(globAllocBits // 8 - frame.byteOffset,
+			offset = make_AwlOffset(globAllocBits // 8 - frame.byteOffset,	#+suffix-u
 						0)
 		globAllocBits += nrBits
 
-		if (((globAllocBits + 7) >> 3) << 3) >= self.maxAllocBits:
+		if (((globAllocBits + 7) >> 3) << 3) >= self.maxAllocBits:		#+suffix-u
 			raise AwlSimError(
 				"Cannot allocate another %d+%d bits on the L-stack. "
 				"The L-stack is exhausted. Maximum size = %d bits." % (
