@@ -2,7 +2,7 @@
 #
 # AWL simulator - status word
 #
-# Copyright 2012-2017 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,8 +29,7 @@ from awlsim.common.cpuspecs import * #+cimport
 from awlsim.common.cpuconfig import *
 from awlsim.common.datatypehelpers import * #+cimport
 from awlsim.common.exceptions import *
-
-from awlsim.core.util import *
+from awlsim.common.util import *
 
 
 class S7StatusWord(object): #+cdef
@@ -161,11 +160,11 @@ class S7StatusWord(object): #+cdef
 	def getByBitNumber(self, bitNumber):					#@nocy
 		try:								#@nocy
 			return self.__bitnr2getter[bitNumber](self)		#@nocy
-		except IndexError as e:						#@nocy
+		except IndexError as e:						#@nocy #@nocov
 			raise AwlSimError("Status word bit fetch '%d' "		#@nocy
 				"out of range" % bitNumber)			#@nocy
 
-#@cy	cdef _Bool getByBitNumber(self, uint8_t bitNumber):
+#@cy	cdef ExBool_t getByBitNumber(self, uint8_t bitNumber) except ExBool_val:
 #@cy		if bitNumber == 0:
 #@cy			return self.NER
 #@cy		elif bitNumber == 1:
@@ -226,26 +225,34 @@ class S7StatusWord(object): #+cdef
 
 	def setForFloatingPoint(self, pyFloat): #@nocy
 #@cy	cdef void setForFloatingPoint(self, double pyFloat):
-		dword = pyFloatToDWord(pyFloat)
-		dwordNoSign = dword & 0x7FFFFFFF
-		if isDenormalPyFloat(pyFloat) or\
-		   (dwordNoSign < 0x00800000 and dwordNoSign != 0):
+		if isDenormalPyFloat(pyFloat):
 			# denorm
 			self.A1, self.A0, self.OV, self.OS = 0, 0, 1, 1
-		elif dwordNoSign == 0:
+		else:
+			self.setForFloatingPointDWord(pyFloatToDWord(pyFloat))
+
+	def setForFloatingPointDWord(self, dword): #@nocy
+#@cy	cdef void setForFloatingPointDWord(self, uint32_t dword):
+#@cy		cdef uint32_t dwordNoSign
+
+		dwordNoSign = dword & 0x7FFFFFFF			#+suffix-u
+		if dwordNoSign == 0:					#+suffix-u
 			# zero
 			self.A1, self.A0, self.OV = 0, 0, 0
-		elif dwordNoSign >= 0x7F800000:
-			if dwordNoSign == 0x7F800000:
+		elif dwordNoSign < 0x00800000:				#+suffix-u
+			# denorm
+			self.A1, self.A0, self.OV, self.OS = 0, 0, 1, 1
+		elif dwordNoSign >= 0x7F800000:				#+suffix-u
+			if dwordNoSign == 0x7F800000:			#+suffix-u
 				# inf
-				if dword & 0x80000000:
+				if dword & 0x80000000:			#+suffix-u
 					self.A1, self.A0, self.OV, self.OS = 0, 1, 1, 1
 				else:
 					self.A1, self.A0, self.OV, self.OS = 1, 0, 1, 1
 			else:
 				# nan
 				self.A1, self.A0, self.OV, self.OS = 1, 1, 1, 1
-		elif dword & 0x80000000:
+		elif dword & 0x80000000:				#+suffix-u
 			# norm neg
 			self.A1, self.A0, self.OV = 0, 1, 0
 		else:
@@ -263,5 +270,5 @@ class S7StatusWord(object): #+cdef
 			ret.append("%s:%d" % (name, self.getByBitNumber(i)))
 		return '  '.join(ret)
 
-	def __repr__(self):
+	def __repr__(self): #@nocov
 		return self.getString(S7CPUConfig.MNEMONICS_DE)
