@@ -24,17 +24,17 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 from awlsim.common.compat import *
 
 from awlsim.common.exceptions import *
+from awlsim.common.util import *
 
 from awlsim.core.systemblocks.systemblocks import * #+cimport
 from awlsim.core.blockinterface import *
 from awlsim.core.datatypes import *
-from awlsim.core.util import *
 
 
 class SFCm4(SFC): #+cdef
 	name = (-4, "__CLKRST", None)
 
-	def run(self): #+cpdef
+	def run(self): #+cpdef #@nocov
 #@cy		cdef S7StatusWord s
 
 		s = self.cpu.statusWord
