@@ -2,7 +2,7 @@
 #
 # AWL simulator - PLC core server client
 #
-# Copyright 2013-2017 Michael Buesch <m@bues.ch>
+# Copyright 2013-2019 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@ from awlsim.coreclient.sshtunnel import *
 from awlsim.coreclient.util import *
 
 from awlsim.coreserver.server import *
+from awlsim.coreserver.messages import *
+from awlsim.coreserver.memarea import *
 
 import sys
 import socket
@@ -67,6 +69,7 @@ class AwlSimClient(object):
 		self.serverProcessPort = None
 		self.__transceiver = None
 		self.__defaultTimeout = 3.0
+		self.__timeoutFactor = 2.0 if isPyPy else 1.0
 		self.__msgWaiters = []
 
 	def spawnServer(self,
@@ -185,9 +188,7 @@ class AwlSimClient(object):
 
 		if self.__transceiver:
 			with contextlib.suppress(AwlSimError, MaintenanceRequest):
-				msg = AwlSimMessage_SHUTDOWN()
-				status = self.__sendAndWaitFor_REPLY(msg)
-				if status != AwlSimMessage_REPLY.STAT_OK:
+				if not self.shutdownCoreServer():
 					printError("AwlSimClient: Failed to shut "
 						"down server via message")
 
@@ -203,9 +204,10 @@ class AwlSimClient(object):
 			    timeout=3.0):
 		"""Connect to a AwlSim-core server.
 		host -> The hostname or IP address to connect to.
-		port -> The port to connect to."""
-
+		port -> The port to connect to.
+		"""
 		self.__defaultTimeout = timeout
+		timeout *= self.__timeoutFactor
 		startTime = monotonic_time()
 		readableSockaddr = host
 		sock, ok = None, False
@@ -225,8 +227,8 @@ class AwlSimClient(object):
 				try:
 					sock.connect(sockaddr)
 				except _SocketErrors as e:
-					if e.errno == errno.ECONNREFUSED or\
-					   e.errno == errno.ENOENT:
+					if (excErrno(e) == errno.ECONNREFUSED or
+					    excErrno(e) == errno.ENOENT):
 						self.sleep(0.1)
 						continue
 					if isJython and\
@@ -243,7 +245,8 @@ class AwlSimClient(object):
 		finally:
 			if not ok and sock:
 				with suppressAllExc:
-					sock.shutdown(socket.SHUT_RDWR)
+					if hasattr(sock, "shutdown"):
+						sock.shutdown(socket.SHUT_RDWR)
 				with suppressAllExc:
 					sock.close()
 		printInfo("AwlSimClient: Connected.")
@@ -253,7 +256,7 @@ class AwlSimClient(object):
 		# Ping the server
 		try:
 			self.__transceiver.send(AwlSimMessage_PING())
-			msg = self.__transceiver.receive(timeout = timeout)
+			msg = self.__transceiver.receive(timeout=timeout)
 			if not msg:
 				raise AwlSimError("AwlSimClient: Server did not "
 					"respond to PING request.")
@@ -316,6 +319,12 @@ class AwlSimClient(object):
 	def __rx_CPUDUMP(self, msg):
 		self.handle_CPUDUMP(msg.dumpText)
 
+	def handle_CPUSTATS(self, msg):
+		pass # Don't do anything by default
+
+	def __rx_CPUSTATS(self, msg):
+		self.handle_CPUSTATS(msg)
+
 	def __rx_MAINTREQ(self, msg):
 		raise msg.maintRequest
 
@@ -361,6 +370,7 @@ class AwlSimClient(object):
 		AwlSimMessage.MSG_ID_CPUCONF		: __rx_NOP,
 		AwlSimMessage.MSG_ID_RUNSTATE		: __rx_NOP,
 		AwlSimMessage.MSG_ID_CPUDUMP		: __rx_CPUDUMP,
+		AwlSimMessage.MSG_ID_CPUSTATS		: __rx_CPUSTATS,
 		AwlSimMessage.MSG_ID_MEMORY		: __rx_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE		: __rx_INSNSTATE,
 	}
@@ -372,6 +382,8 @@ class AwlSimClient(object):
 	def processMessages(self, timeout=None):
 		if not self.__transceiver:
 			return False
+		if timeout is not None:
+			timeout *= self.__timeoutFactor
 		try:
 			msg = self.__transceiver.receive(timeout)
 		except TransferError as e:
@@ -380,7 +392,7 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: "
 				"I/O error in connection to server '%s':\n"
 				"%s (errno = %s)" %\
-				(self.__transceiver.peerInfoString, str(e), str(e.errno)))
+				(self.__transceiver.peerInfoString, str(e), str(excErrno(e))))
 		except TransferError as e:
 			raise AwlSimError("AwlSimClient: "
 				"Connection to server '%s' died. "
@@ -413,18 +425,21 @@ class AwlSimClient(object):
 				"Send error in connection to server '%s':\n"
 				"%s (errno = %s)" %\
 				(self.__transceiver.peerInfoString,
-				 str(e), str(e.errno)))
+				 str(e), str(excErrno(e))))
 
 	def __sendAndWait(self, txMsg, checkRxMsg,
-			  waitTimeout=None,
+			  minTimeout=None,
 			  ignoreMaintenanceRequests=False):
 		waiter = MsgWaiter(checkRxMsg)
 		self.__msgWaiters.append(waiter)
 		try:
 			self.__send(txMsg)
 			now = monotonic_time()
-			end = now + (self.__defaultTimeout if waitTimeout is None\
-				     else waitTimeout)
+			timeout = self.__defaultTimeout
+			if minTimeout is not None:
+				timeout = max(timeout, minTimeout)
+			timeout *= self.__timeoutFactor
+			end = now + timeout
 			while now < end:
 				try:
 					if self.processMessages(0.1):
@@ -438,14 +453,17 @@ class AwlSimClient(object):
 		finally:
 			self.__msgWaiters.remove(waiter)
 
-	def __sendAndWaitFor_REPLY(self, msg, timeout=None,
+	def __sendAndWaitFor_REPLY(self, msg,
+				   minTimeout=None,
 				   ignoreMaintenanceRequests=False):
 		def checkRxMsg(rxMsg):
-			return rxMsg.msgId == AwlSimMessage.MSG_ID_REPLY and\
-			       rxMsg.inReplyToId == msg.msgId and\
-			       rxMsg.inReplyToSeq == msg.seq
-		return self.__sendAndWait(msg, checkRxMsg, timeout,
-					  ignoreMaintenanceRequests).status
+			return (rxMsg.msgId == AwlSimMessage.MSG_ID_REPLY and
+				rxMsg.isReplyTo(msg))
+		waiter = self.__sendAndWait(txMsg=msg,
+					    checkRxMsg=checkRxMsg,
+					    minTimeout=minTimeout,
+					    ignoreMaintenanceRequests=ignoreMaintenanceRequests)
+		return waiter.status
 
 	def reset(self):
 		if not self.__transceiver:
@@ -476,7 +494,8 @@ class AwlSimClient(object):
 			return False
 		msg = AwlSimMessage_GET_RUNSTATE()
 		rxMsg = self.__sendAndWait(msg,
-			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_RUNSTATE)
+			lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_RUNSTATE and
+				       rxMsg.isReplyTo(msg)))
 		if rxMsg.runState == AwlSimMessage_RUNSTATE.STATE_RUN:
 			return True
 		return False
@@ -487,7 +506,8 @@ class AwlSimClient(object):
 		msg = AwlSimMessage_GET_AWLSRC(identHash)
 		if sync:
 			rxMsg = self.__sendAndWait(msg,
-				lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_AWLSRC)
+				lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_AWLSRC and
+					       rxMsg.isReplyTo(msg)))
 			return rxMsg.source
 		else:
 			self.__send(msg)
@@ -499,7 +519,7 @@ class AwlSimClient(object):
 		msg = AwlSimMessage_AWLSRC(awlSource)
 		self.__transceiver.txCork(True)
 		try:
-			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+			status = self.__sendAndWaitFor_REPLY(msg, minTimeout=10.0)
 		finally:
 			self.__transceiver.txCork(False)
 		if status != AwlSimMessage_REPLY.STAT_OK:
@@ -512,7 +532,7 @@ class AwlSimClient(object):
 		msg = AwlSimMessage_FUPSRC(fupSource)
 		self.__transceiver.txCork(True)
 		try:
-			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+			status = self.__sendAndWaitFor_REPLY(msg, minTimeout=10.0)
 		finally:
 			self.__transceiver.txCork(False)
 		if status != AwlSimMessage_REPLY.STAT_OK:
@@ -525,7 +545,7 @@ class AwlSimClient(object):
 		msg = AwlSimMessage_KOPSRC(kopSource)
 		self.__transceiver.txCork(True)
 		try:
-			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+			status = self.__sendAndWaitFor_REPLY(msg, minTimeout=10.0)
 		finally:
 			self.__transceiver.txCork(False)
 		if status != AwlSimMessage_REPLY.STAT_OK:
@@ -550,7 +570,8 @@ class AwlSimClient(object):
 		msg = AwlSimMessage_GET_SYMTABSRC(identHash)
 		if sync:
 			rxMsg = self.__sendAndWait(msg,
-				lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_SYMTABSRC)
+				lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_SYMTABSRC and
+					       rxMsg.isReplyTo(msg)))
 			return rxMsg.source
 		else:
 			self.__send(msg)
@@ -650,17 +671,20 @@ class AwlSimClient(object):
 	# Request the compiled block info from the CPU.
 	# This method is asynchronous.
 	# The idents are returned via handle_BLOCKINFO()
-	def requestBlockInfo(self, reqOBInfo = False,
-			     reqFCInfo = False,
-			     reqFBInfo = False,
-			     reqDBInfo = False):
+	def requestBlockInfo(self,
+			     reqOBInfo=False,
+			     reqFCInfo=False,
+			     reqFBInfo=False,
+			     reqDBInfo=False,
+			     reqUDTInfo=False):
 		if not self.__transceiver:
 			return False
 		self.__send(AwlSimMessage_GET_BLOCKINFO(
-			(AwlSimMessage_GET_BLOCKINFO.GET_OB_INFO if reqOBInfo else 0) |\
-			(AwlSimMessage_GET_BLOCKINFO.GET_FC_INFO if reqFCInfo else 0) |\
-			(AwlSimMessage_GET_BLOCKINFO.GET_FB_INFO if reqFBInfo else 0) |\
-			(AwlSimMessage_GET_BLOCKINFO.GET_DB_INFO if reqDBInfo else 0)))
+			(AwlSimMessage_GET_BLOCKINFO.GET_OB_INFO if reqOBInfo else 0) |
+			(AwlSimMessage_GET_BLOCKINFO.GET_FC_INFO if reqFCInfo else 0) |
+			(AwlSimMessage_GET_BLOCKINFO.GET_FB_INFO if reqFBInfo else 0) |
+			(AwlSimMessage_GET_BLOCKINFO.GET_DB_INFO if reqDBInfo else 0) |
+			(AwlSimMessage_GET_BLOCKINFO.GET_UDT_INFO if reqUDTInfo else 0)))
 		return True
 
 	def __setOption(self, name, value, sync=True):
@@ -692,19 +716,24 @@ class AwlSimClient(object):
 	# fromLine, toLine is the range of AWL line numbers for which
 	# dumping is enabled.
 	def setInsnStateDump(self, enable=True,
-			     sourceId=0, fromLine=1, toLine=0x7FFFFFFF,
+			     sourceId=b"", fromLine=1, toLine=0x7FFFFFFF,
+			     ob1Div=1,
+			     userData=0,
 			     sync=True):
 		if not self.__transceiver:
 			return None
 		msg = AwlSimMessage_INSNSTATE_CONFIG(
-			flags = 0,
-			sourceId = sourceId,
-			fromLine = fromLine,
-			toLine = toLine)
+			flags=0,
+			sourceId=sourceId,
+			fromLine=fromLine,
+			toLine=toLine,
+			ob1Div=ob1Div,
+			userData=userData)
 		if enable:
 			msg.flags |= msg.FLG_CLEAR
+			msg.flags |= msg.FLG_SET
 		else:
-			msg.flags |= msg.FLG_CLEAR_ONLY
+			msg.flags |= msg.FLG_CLEAR
 		if sync:
 			msg.flags |= msg.FLG_SYNC
 			status = self.__sendAndWaitFor_REPLY(msg)
@@ -718,7 +747,8 @@ class AwlSimClient(object):
 			return None
 		msg = AwlSimMessage_GET_CPUSPECS()
 		rxMsg = self.__sendAndWait(msg,
-			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_CPUSPECS)
+			lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_CPUSPECS and
+				       rxMsg.isReplyTo(msg)))
 		return rxMsg.cpuspecs
 
 	def setCpuSpecs(self, cpuspecs):
@@ -735,7 +765,8 @@ class AwlSimClient(object):
 			return None
 		msg = AwlSimMessage_GET_CPUCONF()
 		rxMsg = self.__sendAndWait(msg,
-			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_CPUCONF)
+			lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_CPUCONF and
+				       rxMsg.isReplyTo(msg)))
 		return rxMsg.cpuconf
 
 	def setCpuConf(self, cpuconf):
@@ -810,3 +841,87 @@ class AwlSimClient(object):
 		else:
 			self.__send(msg)
 		return True
+
+	def getCpuStats(self, sync=False):
+		"""Get CPU statistics.
+		This returns AwlSimMessage_CPUSTATS, if sync=True.
+		Otherwise handle_CPUSTATS() is called upon reception of cpustats.
+		Returns None, if an error occurred.
+		"""
+		if not self.__transceiver:
+			return None
+		msg = AwlSimMessage_GET_CPUSTATS()
+		if sync:
+			rxMsg = self.__sendAndWait(msg,
+				lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_CPUSTATS,
+					       rxMsg.isReplyTo(msg)))
+			return rxMsg
+		else:
+			self.__send(msg)
+		return True
+
+	def measStart(self, sync=True):
+		"""Start instruction time measurements.
+		"""
+		if not self.__transceiver:
+			return None
+		msg = AwlSimMessage_MEAS_CONFIG(
+			flags=AwlSimMessage_MEAS_CONFIG.FLG_ENABLE)
+		if sync:
+			rxMsg = self.__sendAndWait(msg,
+				lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_MEAS,
+					       rxMsg.isReplyTo(msg)))
+			if rxMsg.flags & AwlSimMessage_MEAS.FLG_FAIL:
+				return False
+		else:
+			self.__send(msg)
+		return True
+
+	def measStop(self, csv=True, sync=True):
+		"""Stop instruction time measurements and
+		return the measurement report data string, if any.
+		"""
+		if not self.__transceiver:
+			return None
+		msg = AwlSimMessage_MEAS_CONFIG(
+			flags=AwlSimMessage_MEAS_CONFIG.FLG_GETMEAS)
+		if csv:
+			msg.flags |= AwlSimMessage_MEAS_CONFIG.FLG_CSV
+		if sync:
+			rxMsg = self.__sendAndWait(msg,
+				lambda rxMsg: (rxMsg.msgId == AwlSimMessage.MSG_ID_MEAS,
+					       rxMsg.isReplyTo(msg)))
+			if ((rxMsg.flags & AwlSimMessage_MEAS.FLG_FAIL) or
+			    not (rxMsg.flags & AwlSimMessage_MEAS.FLG_HAVEDATA)):
+				return ""
+			return rxMsg.reportStr
+		else:
+			self.__send(msg)
+		return ""
+
+	def shutdownCoreServer(self):
+		"""Shut down the core server.
+		"""
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_SHUTDOWN(AwlSimMessage_SHUTDOWN.SHUTDOWN_CORE)
+		status = self.__sendAndWaitFor_REPLY(msg)
+		return status == AwlSimMessage_REPLY.STAT_OK
+
+	def shutdownCoreServerSystem(self):
+		"""Shut down the core server and the system it runs on.
+		"""
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_SHUTDOWN(AwlSimMessage_SHUTDOWN.SHUTDOWN_SYSTEM_HALT)
+		self.__send(msg)
+		return True
+
+	def rebootCoreServerSystem(self):
+		"""Reboot the core server and the system it runs on.
+		"""
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_SHUTDOWN(AwlSimMessage_SHUTDOWN.SHUTDOWN_SYSTEM_REBOOT)
+		self.__send(msg)
+		return True
