@@ -2,7 +2,7 @@
 #
 # AWL simulator - PLC core server memory area helpers
 #
-# Copyright 2013-2014 Michael Buesch <m@bues.ch>
+# Copyright 2013-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,8 +26,15 @@ from awlsim.common.compat import *
 from awlsim.common.enumeration import *
 from awlsim.common.util import *
 from awlsim.common.exceptions import *
+from awlsim.common.wordpacker import *
 
-import struct
+from awlsim.core.cpu import * #+cimport
+from awlsim.core.memory import * #+cimport
+from awlsim.core.offset import * #+cimport
+from awlsim.core.datablocks import * #+cimport
+from awlsim.core.timers import * #+cimport
+from awlsim.core.counters import * #+cimport
+from awlsim.core.statusword import * #+cimport
 
 
 class MemoryArea(object):
@@ -47,8 +54,6 @@ class MemoryArea(object):
 	FLG_ERR_READ	= 0x01
 	FLG_ERR_WRITE	= 0x02
 
-	_dwordStruct = struct.Struct(str(">I"))
-
 	def __init__(self, memType, flags, index, start, length, data=b''):
 		self.memType = memType
 		self.flags = flags
@@ -71,46 +76,49 @@ class MemoryArea(object):
 		self.flags |= self.FLG_ERR_WRITE
 		raise exception
 
-	def __read_E(self, cpu):
-		dataBytes = cpu.inputs.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseReadErr(
-				AwlSimError("MemoryArea: Read range error")
-			)
-		self.data = dataBytes[self.start : end]
+	def __memoryRead(self, memory): #@nocy
+#@cy	def __memoryRead(self, AwlMemory memory):
+#@cy		cdef AwlMemoryObject memObj
 
-	def __read_A(self, cpu):
-		dataBytes = cpu.outputs.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseReadErr(
-				AwlSimError("MemoryArea: Read range error")
-			)
-		self.data = dataBytes[self.start : end]
-
-	def __read_M(self, cpu):
-		dataBytes = cpu.flags.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseReadErr(
-				AwlSimError("MemoryArea: Read range error")
-			)
-		self.data = dataBytes[self.start : end]
-
-	def __read_L(self, cpu):
-		dataBytes = cpu.callStackTop.localdata.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseReadErr(
-				AwlSimError("MemoryArea: Read range error")
-			)
-		self.data = dataBytes[self.start : end]
-
-	def __read_DB(self, cpu):
 		try:
-			db = cpu.dbs[self.index]
-		except KeyError:
+			if (not (0 <= self.start <= 0xFFFF) or
+			    self.length not in (1, 2, 4)):
+				raise ValueError
+			memObj = memory.fetch(make_AwlOffset(self.start, 0),
+					      self.length * 8)
+		except (AwlSimError, ValueError) as e:
+			self.__raiseReadErr(
+				AwlSimError("MemoryArea: Read range error")
+			)
+		self.data = AwlMemoryObject_asBytes(memObj)[:]
+
+	def __read_E(self, cpu): #@nocy
+#@cy	def __read_E(self, S7CPU cpu):
+		self.__memoryRead(cpu.inputs)
+
+	def __read_A(self, cpu): #@nocy
+#@cy	def __read_A(self, S7CPU cpu):
+		self.__memoryRead(cpu.outputs)
+
+	def __read_M(self, cpu): #@nocy
+#@cy	def __read_M(self, S7CPU cpu):
+		self.__memoryRead(cpu.flags)
+
+	def __read_L(self, cpu): #@nocy
+#@cy	def __read_L(self, S7CPU cpu):
+		#TODO use self.index to select which L-stack we want to access.
+		if not cpu.activeLStack:
+			self.__raiseReadErr(
+				AwlSimError("MemoryArea: No active L-stack")
+			)
+		self.__memoryRead(cpu.activeLStack.memory)
+
+	def __read_DB(self, cpu): #@nocy
+#@cy	def __read_DB(self, S7CPU cpu):
+#@cy		cdef DB db
+
+		db = cpu.getDB(self.index)
+		if not db:
 			self.__raiseReadErr(
 				AwlSimError("MemoryArea: Read access to "
 				"nonexistent DB %d" % self.index)
@@ -120,15 +128,12 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Read access to "
 				"read-protected DB %d" % self.index)
 			)
-		dataBytes = db.structInstance.memory.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseReadErr(
-				AwlSimError("MemoryArea: Read range error")
-			)
-		self.data = dataBytes[self.start : end]
+		self.__memoryRead(db.structInstance.memory)
+
+	def __read_T(self, cpu): #@nocy
+#@cy	def __read_T(self, S7CPU cpu):
+#@cy		cdef Timer timer
 
-	def __read_T(self, cpu):
 		try:
 			timer = cpu.timers[self.index]
 		except IndexError as e:
@@ -136,9 +141,13 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Invalid timer index %d" % self.index)
 			)
 		v = (timer.get() << 31) | timer.getTimevalS5TwithBase()
-		self.data, self.length = self._dwordStruct.pack(v), 4
+		self.data = WordPacker.toBytes(bytearray(4), 32, 0, v)
+		self.length = 4
+
+	def __read_Z(self, cpu): #@nocy
+#@cy	def __read_Z(self, S7CPU cpu):
+#@cy		cdef Counter counter
 
-	def __read_Z(self, cpu):
 		try:
 			counter = cpu.counters[self.index]
 		except IndexError as e:
@@ -146,10 +155,15 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Invalid counter index %d" % self.index)
 			)
 		v = (counter.get() << 31) | counter.getValueBCD()
-		self.data, self.length = self._dwordStruct.pack(v), 4
+		self.data = WordPacker.toBytes(bytearray(4), 32, 0, v)
+		self.length = 4
 
-	def __read_STW(self, cpu):
-		stw = cpu.statusWord.getWord()
+	def __read_STW(self, cpu): #@nocy
+#@cy	def __read_STW(self, S7CPU cpu):
+#@cy		cdef S7StatusWord s
+
+		s = cpu.statusWord
+		stw = s.getWord()
 		self.data, self.length = bytearray(((stw >> 8) & 0xFF, stw & 0xFF)), 2
 
 	__readHandlers = {
@@ -163,37 +177,42 @@ class MemoryArea(object):
 		TYPE_STW	: __read_STW,
 	}
 
-	def __write_E(self, cpu):
-		dataBytes = cpu.inputs.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseWriteErr(
-				AwlSimError("MemoryArea: Write range error")
-			)
-		dataBytes[self.start : end] = self.data
+	def __memoryWrite(self, memory): #@nocy
+#@cy	def __memoryWrite(self, AwlMemory memory):
+#@cy		cdef AwlMemoryObject memObj
 
-	def __write_A(self, cpu):
-		dataBytes = cpu.outputs.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseWriteErr(
-				AwlSimError("MemoryArea: Write range error")
-			)
-		dataBytes[self.start : end] = self.data
-
-	def __write_M(self, cpu):
-		dataBytes = cpu.flags.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseWriteErr(
-				AwlSimError("MemoryArea: Write range error")
-			)
-		dataBytes[self.start : end] = self.data
-
-	def __write_DB(self, cpu):
 		try:
-			db = cpu.dbs[self.index]
-		except KeyError:
+			if (not (0 <= self.start <= 0xFFFF) or
+			    self.length not in (1, 2, 4) or
+			    self.length != len(self.data)):
+				raise ValueError
+			memObj = make_AwlMemoryObject_fromBytes(self.data,
+								self.length * 8)
+			memory.store(make_AwlOffset(self.start, 0),
+				     memObj)
+		except (AwlSimError, ValueError) as e:
+			self.__raiseWriteErr(
+				AwlSimError("MemoryArea: Write range error")
+			)
+
+	def __write_E(self, cpu): #@nocy
+#@cy	def __write_E(self, S7CPU cpu):
+		self.__memoryWrite(cpu.inputs)
+
+	def __write_A(self, cpu): #@nocy
+#@cy	def __write_A(self, S7CPU cpu):
+		self.__memoryWrite(cpu.outputs)
+
+	def __write_M(self, cpu): #@nocy
+#@cy	def __write_M(self, S7CPU cpu):
+		self.__memoryWrite(cpu.flags)
+
+	def __write_DB(self, cpu): #@nocy
+#@cy	def __write_DB(self, S7CPU cpu):
+#@cy		cdef DB db
+
+		db = cpu.getDB(self.index)
+		if not db:
 			self.__raiseWriteErr(
 				AwlSimError("MemoryArea: Write access to "
 				"nonexistent DB %d" % self.index)
@@ -203,15 +222,12 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Write access to "
 				"write-protected DB %d" % self.index)
 			)
-		dataBytes = db.structInstance.memory.dataBytes
-		end = self.start + self.length
-		if end > len(dataBytes):
-			self.__raiseWriteErr(
-				AwlSimError("MemoryArea: Write range error")
-			)
-		dataBytes[self.start : end] = self.data
+		self.__memoryWrite(db.structInstance.memory)
+
+	def __write_T(self, cpu): #@nocy
+#@cy	def __write_T(self, S7CPU cpu):
+#@cy		cdef Timer timer
 
-	def __write_T(self, cpu):
 		try:
 			timer = cpu.timers[self.index]
 		except IndexError as e:
@@ -219,16 +235,20 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Invalid timer index %d" % self.index)
 			)
 		try:
-			(dword, ) = self._dwordStruct.unpack(self.data)
-			if dword > 0xFFFF:
+			if (self.length not in (2, 4) or
+			    self.length != len(self.data)):
 				raise ValueError
-			timer.setTimevalS5T(dword)
-		except (struct.error, ValueError, AwlSimError) as e:
+			dword = WordPacker.fromBytes(self.data, self.length * 8, 0)
+			timer.setTimevalS5T(dword & 0xFFFF)
+		except (ValueError, AwlSimError) as e:
 			self.__raiseWriteErr(
 				AwlSimError("MemoryArea: Timer value error")
 			)
 
-	def __write_Z(self, cpu):
+	def __write_Z(self, cpu): #@nocy
+#@cy	def __write_Z(self, S7CPU cpu):
+#@cy		cdef Counter counter
+
 		try:
 			counter = cpu.counters[self.index]
 		except IndexError as e:
@@ -236,11 +256,12 @@ class MemoryArea(object):
 				AwlSimError("MemoryArea: Invalid counter index %d" % self.index)
 			)
 		try:
-			(dword, ) = self._dwordStruct.unpack(self.data)
-			if dword > 0xFFFF:
+			if (self.length not in (2, 4) or
+			    self.length != len(self.data)):
 				raise ValueError
-			counter.setValueBCD(dword)
-		except (struct.error, ValueError, AwlSimError) as e:
+			dword = WordPacker.fromBytes(self.data, self.length * 8, 0)
+			counter.setValueBCD(dword & 0xFFFF)
+		except (ValueError, AwlSimError) as e:
 			self.__raiseWriteErr(
 				AwlSimError("MemoryArea: Counter value error")
 			)
