@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI CPU configuration widget
 #
-# Copyright 2012-2018 Michael Buesch <m@bues.ch>
+# Copyright 2012-2020 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -164,7 +164,7 @@ class CpuConfigWidget(QWidget):
 		self.callStackSpinBox.setToolTip(label.toolTip())
 		group.layout().addWidget(self.callStackSpinBox, 8, 1)
 
-		self.layout().addWidget(group, 0, 0, 2, 1)
+		self.layout().addWidget(group, 0, 0, 3, 1)
 
 		group = QGroupBox("Hardware configuration", self)
 		group.setLayout(QGridLayout())
@@ -187,7 +187,9 @@ class CpuConfigWidget(QWidget):
 
 		label = QLabel("Cycle (OB 1) time limit", self)
 		label.setToolTip(
-			"Time limit of one cycle (OB 1) run, in milliseconds.")
+			"Time limit of one cycle (OB 1) run, in milliseconds.\n"
+			"The CPU goes into exceptional state (STOP), if the\n"
+			"cycle time limit is exceeded.")
 		group.layout().addWidget(label, 2, 0)
 		self.cycleTimeSpinBox = QDoubleSpinBox(self)
 		self.cycleTimeSpinBox.setToolTip(label.toolTip())
@@ -198,6 +200,27 @@ class CpuConfigWidget(QWidget):
 		self.cycleTimeSpinBox.setDecimals(1)
 		group.layout().addWidget(self.cycleTimeSpinBox, 2, 1)
 
+		label = QLabel("Cycle (OB 1) time minimum", self)
+		label.setToolTip(
+			"Cycle time minimum target of one cycle (OB 1) run, in milliseconds.\n"
+			"If this value is bigger than zero, then the CPU increases\n"
+			"the cycle time to approximately this value.\n"
+			"It does so by sleeping at the end of the cycle (OB 1).\n"
+			"This decreases power consumption.\n"
+			"But it also increases the OB 1 latency to the specified time.\n"
+			"If the actual cycle time needed for calculations is bigger\n"
+			"than this value, no sleep will happen.\n"
+			"A value in the region of 10 ms is recommended.")
+		group.layout().addWidget(label, 3, 0)
+		self.cycleTimeTargetSpinBox = QDoubleSpinBox(self)
+		self.cycleTimeTargetSpinBox.setToolTip(label.toolTip())
+		self.cycleTimeTargetSpinBox.setSuffix(" ms")
+		self.cycleTimeTargetSpinBox.setSingleStep(1.0)
+		self.cycleTimeTargetSpinBox.setMinimum(0.0)
+		self.cycleTimeTargetSpinBox.setMaximum(1000.0)
+		self.cycleTimeTargetSpinBox.setDecimals(1)
+		group.layout().addWidget(self.cycleTimeTargetSpinBox, 3, 1)
+
 		self.layout().addWidget(group, 0, 1, 1, 1)
 
 		group = QGroupBox("AWL language", self)
@@ -228,11 +251,24 @@ class CpuConfigWidget(QWidget):
 		group.layout().setRowStretch(2, 1)
 		self.layout().addWidget(group, 1, 1, 1, 1)
 
+		group = QGroupBox("Program", self)
+		group.setLayout(QGridLayout())
+
+		self.preDownloadValidationCheckBox = QCheckBox(
+			"Enable pre-download program validation", self)
+		self.preDownloadValidationCheckBox.setToolTip(
+			"Validate program before downloading it to the CPU.\n"
+			"This may catch certain program errors before download.")
+		group.layout().addWidget(self.preDownloadValidationCheckBox, 0, 0, 1, 1)
+
+		self.layout().addWidget(group, 2, 1, 1, 1)
+
 		self.layout().setRowStretch(2, 1)
 
 	def loadFromProject(self, project):
 		specs = project.getCpuSpecs()
 		conf = project.getCpuConf()
+		guiSettings = project.getGuiSettings()
 
 		index = self.accuCombo.findData(specs.nrAccus)
 		self.accuCombo.setCurrentIndex(index if index >= 0 else 0)
@@ -255,10 +291,15 @@ class CpuConfigWidget(QWidget):
 		self.extInsnsCheckBox.setCheckState(
 			Qt.Checked if conf.extInsnsEn else Qt.Unchecked)
 		self.cycleTimeSpinBox.setValue(conf.cycleTimeLimitUs / 1000.0)
+		self.cycleTimeTargetSpinBox.setValue(conf.cycleTimeTargetUs / 1000.0)
+
+		self.preDownloadValidationCheckBox.setCheckState(
+			Qt.Checked if guiSettings.getPreDownloadValidationEn() else Qt.Unchecked)
 
 	def storeToProject(self, project):
 		specs = project.getCpuSpecs()
 		conf = project.getCpuConf()
+		guiSettings = project.getGuiSettings()
 
 		mnemonics = self.mnemonicsCombo.itemData(self.mnemonicsCombo.currentIndex())
 		nrAccus = self.accuCombo.itemData(self.accuCombo.currentIndex())
@@ -274,6 +315,8 @@ class CpuConfigWidget(QWidget):
 		obTempEnabled = self.obTempCheckBox.checkState() == Qt.Checked
 		extInsnsEnabled = self.extInsnsCheckBox.checkState() == Qt.Checked
 		cycleTimeLimit = self.cycleTimeSpinBox.value()
+		cycleTimeTarget = self.cycleTimeTargetSpinBox.value()
+		preDownloadValidation = self.preDownloadValidationCheckBox.checkState() == Qt.Checked
 
 		specs.setNrAccus(nrAccus)
 		specs.setNrTimers(nrTimers)
@@ -289,6 +332,8 @@ class CpuConfigWidget(QWidget):
 		conf.setOBStartinfoEn(obTempEnabled)
 		conf.setExtInsnsEn(extInsnsEnabled)
 		conf.setCycleTimeLimitUs(int(round(cycleTimeLimit * 1000.0)))
+		conf.setCycleTimeTargetUs(int(round(cycleTimeTarget * 1000.0)))
+		guiSettings.setPreDownloadValidationEn(preDownloadValidation)
 
 		return True
 
