@@ -2,7 +2,7 @@
 #
 # AWL simulator - FUP - Grid classes
 #
-# Copyright 2016-2018 Michael Buesch <m@bues.ch>
+# Copyright 2016-2020 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@ from awlsim.common.compat import *
 
 from awlsim.common.xmlfactory import *
 
+from awlsim.awloptimizer.awloptimizer import *
+
 from awlsim.gui.geo2d import *
 from awlsim.gui.fup.fup_base import *
 from awlsim.gui.fup.fup_wire import *
@@ -169,6 +171,7 @@ class FupGrid(FupBaseClass):
 		self.wires = set()		# The FupConnIn/Out()s in this grid
 
 		self.selectedCells = set()	# Set of tuples (gridX, gridY) of selected cells
+		self.erroneousCells = set()	# Set of tuples (gridX, gridY) of erroneous cells
 		self.selectedElems = set()	# Set of selected elements in this grid
 		self.expandedElems = set()	# Set of expanded elements in this grid
 		self.clickedElem = None		# The recently clicked element in this grid
@@ -197,6 +200,7 @@ class FupGrid(FupBaseClass):
 		self.elems = []
 		self.__elemsByUUID = {}
 		self.selectedCells.clear()
+		self.erroneousCells.clear()
 		self.selectedElems.clear()
 		self.collisionCacheClear()
 
@@ -241,6 +245,14 @@ class FupGrid(FupBaseClass):
 			return self.__drawWidget.interfDef
 		return None
 
+	@property
+	def symTabSources(self):
+		"""Get a list of SymTabSource()s.
+		"""
+		if self.__drawWidget:
+			return self.__drawWidget.symTabSources
+		return []
+
 	def getUnusedWireIdNum(self):
 		"""Get an unused wire idNum.
 		"""
@@ -534,6 +546,31 @@ class FupGrid(FupBaseClass):
 		"""
 		return (x, y) in self.selectedCells
 
+	def setCellError(self, x, y, error=True):
+		"""Add (or remove) one cell to the set of erroneous cells.
+		"""
+		if x >= 0 and y >= 0:
+			if error:
+				self.erroneousCells.add((x, y))
+			else:
+				with contextlib.suppress(KeyError):
+					self.erroneousCells.remove((x, y))
+
+	def clearAllCellErrors(self):
+		"""Clear all error markers.
+		"""
+		self.erroneousCells.clear()
+
+	def cellHasError(self, x, y):
+		"""Check if a cell is marked as erroneous.
+		"""
+		return (x, y) in self.erroneousCells
+
+	def haveErroneousCells(self):
+		"""Check if any error marker exists.
+		"""
+		return bool(self.erroneousCells)
+
 	def merge(self, otherGrid,
 		  offsetX=0, offsetY=0,
 		  selectInsertedElems=True):
