@@ -2,7 +2,7 @@
 #
 # Python compatibility
 #
-# Copyright 2012-2016 Michael Buesch <m@bues.ch>
+# Copyright 2012-2020 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,12 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 
 import sys
 
+__all__ = [
+	"isPy3Compat",
+	"isPy2Compat",
+	"isMicropython",
+	"IOError",
+]
 
 # isPy3Compat is True, if the interpreter is Python 3 compatible.
 isPy3Compat = sys.version_info[0] == 3
@@ -30,29 +36,11 @@ isPy3Compat = sys.version_info[0] == 3
 # isPy2Compat is True, if the interpreter is Python 2 compatible.
 isPy2Compat = sys.version_info[0] == 2
 
-# input() compatibility.
-# Force Python3 behavior
-if isPy2Compat:
-	input = raw_input
+# isMicroPython is True, if the interpreter is Micropython
+isMicropython = hasattr(sys, "implementation") and sys.implementation.name == "micropython"
 
-# range() compatibility.
-# Force Python3 behavior
-if isPy2Compat:
-	range = xrange
-
-# reduce() compatibility.
-# Force Python2 behavior
-if isPy3Compat:
-	from functools import reduce
-
-# BlockingIOError dummy
+# IOError dummy
 try:
-	BlockingIOError
+	IOError = IOError
 except NameError:
-	class BlockingIOError(BaseException): pass
-
-# ConnectionError dummy
-try:
-	ConnectionError
-except NameError:
-	ConnectionError = OSError
+	IOError = OSError
