@@ -26,6 +26,10 @@ else:
 	from configparser import ConfigParser as _ConfigParser
 	from configparser import Error as _ConfigParserError
 
+__all__ = [
+	"PbConfError",
+	"PbConf",
+]
 
 class PbConfError(ProfibusError):
 	pass
@@ -37,6 +41,8 @@ class PbConf(object):
 	class _SlaveConf(object):
 		"""Slave configuration.
 		"""
+		index		= None
+		name		= None
 		addr		= None
 		gsd		= None
 		syncMode	= None
@@ -51,7 +57,10 @@ class PbConf(object):
 			"""
 			from pyprofibus.dp_master import DpSlaveDesc
 			slaveDesc = DpSlaveDesc(gsd=self.gsd,
-						slaveAddr=self.addr)
+						slaveAddr=self.addr,
+						slaveConf=self)
+			slaveDesc.index = self.index
+			slaveDesc.name = self.name
 
 			# Create Chk_Cfg telegram
 			slaveDesc.setCfgDataElements(self.gsd.getCfgDataElements())
@@ -86,18 +95,17 @@ class PbConf(object):
 
 	@classmethod
 	def fromFile(cls, filename):
-		try:
-			with open(filename, "rb") as fd:
-				data = fd.read().decode("UTF-8")
-		except (IOError, UnicodeError) as e:
-			raise PbConfError("Failed to read '%s': %s" %\
-				(filename, str(e)))
-		return cls(data, filename)
+		if isPy2Compat:
+			with open(filename, "r") as fd:
+				return cls(fd, filename)
+		else:
+			with open(filename, "r", encoding="UTF-8") as fd:
+				return cls(fd, filename)
 
 	__reSlave = re.compile(r'^SLAVE_(\d+)$')
 	__reMod = re.compile(r'^module_(\d+)$')
 
-	def __init__(self, text, filename = None):
+	def __init__(self, fd, filename=None):
 		def get(section, option, fallback = None):
 			if p.has_option(section, option):
 				return p.get(section, option)
@@ -121,11 +129,10 @@ class PbConf(object):
 			return fallback
 		try:
 			p = _ConfigParser()
-			textIO = StringIO(text)
 			if hasattr(p, "read_file"):
-				p.read_file(textIO, filename)
+				p.read_file(fd, filename)
 			else:
-				p.readfp(textIO, filename)
+				p.readfp(fd, filename)
 
 			# [PROFIBUS]
 			self.debug = getint("PROFIBUS", "debug",
@@ -164,10 +171,22 @@ class PbConf(object):
 				m = self.__reSlave.match(section)
 				if not m:
 					continue
+				index = int(m.group(1))
 				s = self._SlaveConf()
+				s.index = index
+				s.name = get(section, "name", section)
 				s.addr = getint(section, "addr")
-				s.gsd = GsdInterp.fromFile(
-					get(section, "gsd"))
+				try:
+					s.gsd = GsdInterp.fromFile(
+						get(section, "gsd"),
+						debug=(self.debug > 0))
+				except GsdError as origExc:
+					try:
+						s.gsd = GsdInterp.fromPy(
+							get(section, "gsd").replace(".", "_"),
+							debug=(self.debug > 0))
+					except GsdError as e:
+						raise GsdError("%s\n%s" % (str(origExc), str(e)))
 				s.syncMode = getboolean(section, "sync_mode",
 							fallback=False)
 				s.freezeMode = getboolean(section, "freeze_mode",
@@ -189,7 +208,7 @@ class PbConf(object):
 
 				mods = [ o for o in p.options(section)
 					 if self.__reMod.match(o) ]
-				mods.sort(key = lambda o: self.__reMod.match(o).group(1))
+				mods.sort(key = lambda o: int(self.__reMod.match(o).group(1)))
 				if s.gsd.isModular():
 					for option in mods:
 						s.gsd.setConfiguredModule(get(section, option))
@@ -200,6 +219,9 @@ class PbConf(object):
 
 				self.slaveConfs.append(s)
 
+		except (IOError, UnicodeError) as e:
+			raise PbConfError("Failed to read '%s': %s" %\
+				(filename, str(e)))
 		except (_ConfigParserError, ValueError) as e:
 			raise PbConfError("Profibus config file parse "
 				"error:\n%s" % str(e))
