@@ -2,7 +2,7 @@
 #
 # PROFIBUS DP - Layer 7
 #
-# Copyright (c) 2013-2016 Michael Buesch <m@bues.ch>
+# Copyright (c) 2013-2020 Michael Buesch <m@bues.ch>
 #
 # Licensed under the terms of the GNU General Public License version 2,
 # or (at your option) any later version.
@@ -14,26 +14,47 @@ from pyprofibus.compat import *
 from pyprofibus.fdl import *
 from pyprofibus.util import *
 
+__all__ = [
+	"DpError",
+	"DpTransceiver",
+	"DpTelegram",
+	"DpTelegram_DataExchange_Req",
+	"DpTelegram_DataExchange_Con",
+	"DpTelegram_SlaveDiag_Req",
+	"DpTelegram_SlaveDiag_Con",
+	"DpTelegram_SetPrm_Req",
+	"DpCfgDataElement",
+	"DpTelegram_ChkCfg_Req",
+	"DpTelegram_GetCfg_Req",
+	"DpTelegram_GetCfg_Con",
+	"DpTelegram_GlobalControl",
+]
 
 class DpError(ProfibusError):
-	pass
+	__slots__ = (
+	)
 
 class DpTransceiver(object):
+	__slots__ = (
+		"fdlTrans",
+		"thisIsMaster",
+	)
+
 	def __init__(self, fdlTrans, thisIsMaster):
 		self.fdlTrans = fdlTrans
 		self.thisIsMaster = thisIsMaster
 
-	def poll(self, timeout = 0):
+	def poll(self, timeout=0.0):
 		retTelegram = None
 		ok, fdlTelegram = self.fdlTrans.poll(timeout)
 		if ok and fdlTelegram:
-			if fdlTelegram.sd in {FdlTelegram.SD1,
+			if fdlTelegram.sd in (FdlTelegram.SD1,
 					      FdlTelegram.SD2,
-					      FdlTelegram.SD3}:
+					      FdlTelegram.SD3,):
 				retTelegram = DpTelegram.fromFdlTelegram(
 						fdlTelegram, self.thisIsMaster)
-			elif fdlTelegram.sd in {FdlTelegram.SC,
-						FdlTelegram.SD4}:
+			elif fdlTelegram.sd in (FdlTelegram.SC,
+						FdlTelegram.SD4,):
 				retTelegram = fdlTelegram
 			else:
 				ok = False
@@ -64,6 +85,14 @@ class DpTelegram(object):
 	DSAP_SET_PRM		= 61
 	DSAP_CHK_CFG		= 62
 
+	__slots__ = (
+		"da",
+		"sa",
+		"fc",
+		"dsap",
+		"ssap",
+	)
+
 	def __init__(self, da, sa, fc, dsap=None, ssap=None):
 		self.da = da
 		self.sa = sa
@@ -72,19 +101,22 @@ class DpTelegram(object):
 		self.ssap = ssap
 
 	def __repr__(self):
-		return "DpTelegram(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s) => %s" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc), intToHex(self.dsap),
-			 intToHex(self.ssap),
-			 str(self.getDU()))
+		return ("DpTelegram(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s, du=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap),
+			bytesToHex(self.getDU())))
 
 	def toFdlTelegram(self):
 		du = self.getDU()
 
-		dae, sae = [], []
+		dae = bytearray()
 		if self.dsap is not None:
 			dae.append(self.dsap)
+		sae = bytearray()
 		if self.ssap is not None:
 			sae.append(self.ssap)
 
@@ -163,43 +195,53 @@ class DpTelegram(object):
 	# Get Data-Unit.
 	# This function is overloaded in subclasses.
 	def getDU(self):
-		return []
+		return b""
 
 	@classmethod
 	def checkType(cls, telegram):
 		return isinstance(telegram, cls)
 
 class _DataExchange_Common(DpTelegram):
+	__slots__ = (
+		"du",
+	)
+
 	def __init__(self, da, sa, fc, du):
 		DpTelegram.__init__(self,
 			da=da, sa=sa, fc=fc)
-		self.du = list(du[:])
+		self.du = bytearray(du)
 
 	def appendData(self, data):
 		if not self.du:
-			self.du = []
+			self.du = bytearray()
 		self.du.append(data)
 
 	def getDU(self):
-		return self.du[:]
+		return bytearray(self.du)
 
 	@classmethod
 	def fromFdlTelegram(cls, fdl):
 		dp = cls(da=fdl.da,
 			 sa=fdl.sa,
 			 fc=fdl.fc,
-			 du=fdl.du if fdl.du else ())
+			 du=fdl.du if fdl.du else b"")
 		return dp
 
 class DpTelegram_DataExchange_Req(_DataExchange_Common):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SRD_HI |
 		        FdlTelegram.FC_REQ,
-		     du=()):
+		     du=b""):
 		_DataExchange_Common.__init__(self,
 			da=da, sa=sa, fc=fc, du=du)
 
 class DpTelegram_DataExchange_Con(_DataExchange_Common):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_DL,
 		     du=()):
@@ -207,6 +249,9 @@ class DpTelegram_DataExchange_Con(_DataExchange_Common):
 			da=da, sa=sa, fc=fc, du=du)
 
 class DpTelegram_SlaveDiag_Req(DpTelegram):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SRD_HI |
 		        FdlTelegram.FC_REQ,
@@ -248,11 +293,19 @@ class DpTelegram_SlaveDiag_Con(DpTelegram):
 	# Flags byte 2
 	B2_EXTDIAGOVR		= 0x80	# Ext_Diag_Overflow
 
+	__slots__ = (
+		"b0",
+		"b1",
+		"b2",
+		"masterAddr",
+		"identNumber",
+	)
+
 	def __init__(self, da, sa, fc=FdlTelegram.FC_DL,
 		     dsap=DpTelegram.SSAP_MS0,
 		     ssap=DpTelegram.DSAP_SLAVE_DIAG):
 		DpTelegram.__init__(self, da=da, sa=sa, fc=fc,
-			dsap=dsap, ssap=ssap)
+				    dsap=dsap, ssap=ssap)
 		self.b0 = 0
 		self.b1 = 0
 		self.b2 = 0
@@ -260,14 +313,19 @@ class DpTelegram_SlaveDiag_Con(DpTelegram):
 		self.identNumber = 0
 
 	def __repr__(self):
-		return "DpTelegram_SlaveDiag_Con(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s) => " \
-			"(b0=%s, b1=%s, b2=%s, masterAddr=%s, identNumber=%s)" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc),
-			 intToHex(self.dsap), intToHex(self.ssap),
-			 intToHex(self.b0), intToHex(self.b1), intToHex(self.b2),
-			 intToHex(self.masterAddr), intToHex(self.identNumber))
+		return ("DpTelegram_SlaveDiag_Con(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s, "
+			"b0=%s, b1=%s, b2=%s, masterAddr=%s, identNumber=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap),
+			intToHex(self.b0),
+			intToHex(self.b1),
+			intToHex(self.b2),
+			intToHex(self.masterAddr),
+			intToHex(self.identNumber)))
 
 	@classmethod
 	def fromFdlTelegram(cls, fdl):
@@ -287,10 +345,12 @@ class DpTelegram_SlaveDiag_Con(DpTelegram):
 		return dp
 
 	def getDU(self):
-		return [self.b0, self.b1, self.b2,
-			self.masterAddr,
-			(self.identNumber >> 8) & 0xFF,
-			self.identNumber & 0xFF]
+		return bytearray((self.b0,
+				  self.b1,
+				  self.b2,
+				  self.masterAddr,
+				  (self.identNumber >> 8) & 0xFF,
+				  self.identNumber & 0xFF))
 
 	def notExist(self):
 		return (self.b0 & self.B0_STANOEX) != 0
@@ -316,19 +376,20 @@ class DpTelegram_SlaveDiag_Con(DpTelegram):
 	def hasOnebit(self):
 		return (self.b1 & self.B1_ONE) != 0
 
+	def prmReq(self):
+		return (self.b1 & self.B1_PRMREQ) != 0
+
 	def needsNewPrmCfg(self):
-		return ((self.b0 & self.B0_CFGFLT) != 0 or\
-			(self.b0 & self.B0_PRMFLT) != 0 or\
+		return ((self.b0 & self.B0_CFGFLT) != 0 or
+			(self.b0 & self.B0_PRMFLT) != 0 or
 			(self.b1 & self.B1_PRMREQ) != 0)
 
 	def isReadyDataEx(self):
-		return not ((self.b0 & (\
-			     self.B0_STANOEX |\
-			     self.B0_STANORDY |\
-			     self.B0_CFGFLT |\
-			     self.B0_PRMFLT)) != 0 or\
-			    (self.b1 & (\
-			     self.B1_PRMREQ)) != 0)
+		return ((self.b0 & (self.B0_STANOEX |
+				    self.B0_STANORDY |
+				    self.B0_CFGFLT |
+				    self.B0_PRMFLT)) == 0 and
+			(self.b1 & (self.B1_PRMREQ)) == 0)
 
 class DpTelegram_SetPrm_Req(DpTelegram):
 	# Station status
@@ -374,6 +435,16 @@ class DpTelegram_SetPrm_Req(DpTelegram):
 	DPV1PRM2_R6		= 0x40	# Reserved bit 6
 	DPV1PRM2_REDUN		= 0x80	# Redundancy commands on
 
+	__slots__ = (
+		"stationStatus",
+		"wdFact1",
+		"wdFact2",
+		"minTSDR",
+		"identNumber",
+		"groupIdent",
+		"userPrmData",
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SRD_HI |
 		        FdlTelegram.FC_REQ,
@@ -390,19 +461,23 @@ class DpTelegram_SetPrm_Req(DpTelegram):
 		self.clearUserPrmData()
 
 	def __repr__(self):
-		return "DpTelegram_SetPrm_Req(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s) => " \
-			"(stationStatus=%s, wdFact1=%s, wdFact2=%s, " \
-			"minTSDR=%s, identNumber=%s, groupIdent=%s " \
-			"userPrmData=%s)" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc),
-			 intToHex(self.dsap), intToHex(self.ssap),
-			 intToHex(self.stationStatus),
-			 intToHex(self.wdFact1), intToHex(self.wdFact2),
-			 intToHex(self.minTSDR), intToHex(self.identNumber),
-			 intToHex(self.groupIdent),
-			 intListToHex(self.userPrmData))
+		return ("DpTelegram_SetPrm_Req(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s, "
+			"stationStatus=%s, wdFact1=%s, wdFact2=%s, "
+			"minTSDR=%s, identNumber=%s, groupIdent=%s, "
+			"userPrmData=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap),
+			intToHex(self.stationStatus),
+			intToHex(self.wdFact1),
+			intToHex(self.wdFact2),
+			intToHex(self.minTSDR),
+			intToHex(self.identNumber),
+			intToHex(self.groupIdent),
+			bytesToHex(self.userPrmData)))
 
 	@classmethod
 	def fromFdlTelegram(cls, fdl):
@@ -425,18 +500,20 @@ class DpTelegram_SetPrm_Req(DpTelegram):
 		return dp
 
 	def clearUserPrmData(self):
-		self.userPrmData = []
+		self.userPrmData = bytearray()
 
 	def addUserPrmData(self, data):
+		assert isinstance(data, (bytes, bytearray))
 		self.userPrmData.extend(data)
 
 	def getDU(self):
-		du = [self.stationStatus,
-		      self.wdFact1, self.wdFact2,
-		      self.minTSDR,
-		      (self.identNumber >> 8) & 0xFF,
-		      self.identNumber & 0xFF,
-		      self.groupIdent]
+		du = bytearray((self.stationStatus,
+				self.wdFact1,
+				self.wdFact2,
+				self.minTSDR,
+				(self.identNumber >> 8) & 0xFF,
+				self.identNumber & 0xFF,
+				self.groupIdent))
 		du.extend(self.userPrmData)
 		return du
 
@@ -463,21 +540,31 @@ class DpCfgDataElement(object):
 	LEN_WORDS		= 0x40	# Word structure
 	LEN_CON_WHOLE		= 0x80	# Consistency over whole length
 
-	def __init__(self, identifier=0, lengthBytes=()):
+	__slots__ = (
+		"identifier",
+		"lengthBytes",
+	)
+
+	def __init__(self, identifier=0, lengthBytes=b""):
 		self.identifier = identifier
 		self.lengthBytes = lengthBytes
+		assert isinstance(lengthBytes, (bytes, bytearray))
 	
 	def __repr__(self):
-		return "DpCfgDataElement(identifier=%s, length=%s)" %\
-			(intToHex(self.identifier),
-			 intListToHex(self.lengthBytes))
+		return ("DpCfgDataElement(identifier=%s, length=%s)" % (
+			intToHex(self.identifier),
+			bytesToHex(self.lengthBytes)))
 
 	def getDU(self):
-		du = [ self.identifier ]
+		du = bytearray((self.identifier,))
 		du.extend(self.lengthBytes)
 		return du
 
 class DpTelegram_ChkCfg_Req(DpTelegram):
+	__slots__ = (
+		"cfgData",
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SRD_HI |
 		        FdlTelegram.FC_REQ,
@@ -488,13 +575,14 @@ class DpTelegram_ChkCfg_Req(DpTelegram):
 		self.clearCfgDataElements()
 
 	def __repr__(self):
-		return "DpTelegram_ChkCfg_Req(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s) => " \
-			"(%s)" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc),
-			 intToHex(self.dsap), intToHex(self.ssap),
-			 ", ".join(str(d) for d in self.cfgData))
+		return ("DpTelegram_ChkCfg_Req(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s cfgData=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap),
+			str(self.cfgData)))
 
 	def clearCfgDataElements(self):
 		self.cfgData = []
@@ -531,24 +619,32 @@ class DpTelegram_ChkCfg_Req(DpTelegram):
 		return dp
 
 	def getDU(self):
-		du = []
+		du = bytearray()
 		for cfgData in self.cfgData:
 			du.extend(cfgData.getDU())
 		return du
 
 class _Cfg_Common(DpTelegram):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa, fc, dsap, ssap):
 		DpTelegram.__init__(self, da=da, sa=sa, fc=fc,
 				    dsap=dsap, ssap=ssap)
 
 	def __repr__(self):
-		return "_Cfg_Common(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s)" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc),
-			 intToHex(self.dsap), intToHex(self.ssap))
+		return ("_Cfg_Common(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap)))
 
 class DpTelegram_GetCfg_Req(_Cfg_Common):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SRD_HI |
 		        FdlTelegram.FC_REQ,
@@ -562,6 +658,9 @@ class DpTelegram_GetCfg_Req(_Cfg_Common):
 		pass#TODO
 
 class DpTelegram_GetCfg_Con(_Cfg_Common):
+	__slots__ = (
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_DL,
 		     dsap=DpTelegram.SSAP_MS0,
@@ -592,6 +691,11 @@ class DpTelegram_GlobalControl(DpTelegram):
 	GSEL_GROUP7		= 0x40	# Group 7 is addressed
 	GSEL_GROUP8		= 0x80	# Group 8 is addressed
 
+	__slots__ = (
+		"controlCommand",
+		"groupSelect",
+	)
+
 	def __init__(self, da, sa,
 		     fc=FdlTelegram.FC_SDN_HI |
 		        FdlTelegram.FC_REQ,
@@ -603,11 +707,13 @@ class DpTelegram_GlobalControl(DpTelegram):
 		self.groupSelect = self.GSEL_BROADCAST	# Group_Select
 
 	def __repr__(self):
-		return "DpTelegram_GlobalControl(da=%s, sa=%s, fc=%s, " \
-			"dsap=%s, ssap=%s)" %\
-			(intToHex(self.da), intToHex(self.sa),
-			 intToHex(self.fc),
-			 intToHex(self.dsap), intToHex(self.ssap))
+		return ("DpTelegram_GlobalControl(da=%s, sa=%s, fc=%s, "
+			"dsap=%s, ssap=%s)" % (
+			intToHex(self.da),
+			intToHex(self.sa),
+			intToHex(self.fc),
+			intToHex(self.dsap),
+			intToHex(self.ssap)))
 
 	@classmethod
 	def fromFdlTelegram(cls, fdl):
@@ -624,4 +730,5 @@ class DpTelegram_GlobalControl(DpTelegram):
 		return dp
 
 	def getDU(self):
-		return [self.controlCommand, self.groupSelect]
+		return bytearray((self.controlCommand,
+				  self.groupSelect))
