@@ -14,12 +14,20 @@ from pyprofibus.compat import *
 from pyprofibus.phy import *
 from pyprofibus.fdl import *
 from pyprofibus.dp import *
+from pyprofibus.util import *
 
+__all__ = [
+	"CpPhyDummySlave",
+]
 
 class CpPhyDummySlave(CpPhy):
 	"""Dummy slave PROFIBUS CP PHYsical layer
 	"""
 
+	__slots__ = (
+		"__pollQueue",
+	)
+
 	def __init__(self, *args, **kwargs):
 		super(CpPhyDummySlave, self).__init__(*args, **kwargs)
 		self.__pollQueue = []
@@ -42,10 +50,10 @@ class CpPhyDummySlave(CpPhy):
 					       bytesToHex(telegramData)))
 		self.__mockSend(telegramData, srd = srd)
 
-	def pollData(self, timeout = 0):
+	def pollData(self, timeout=0.0):
 		"""Poll received data from the physical line.
 		timeout => timeout in seconds.
-			   0 = no timeout, return immediately.
+			   0.0 = no timeout, return immediately.
 			   negative = unlimited.
 		"""
 		try:
