@@ -2,26 +2,46 @@
 #
 # Utility helpers
 #
-# Copyright (c) 2013-2016 Michael Buesch <m@bues.ch>
+# Copyright (c) 2013-2020 Michael Buesch <m@bues.ch>
 #
 # Licensed under the terms of the GNU General Public License version 2,
 # or (at your option) any later version.
 #
 
+from __future__ import division, absolute_import, print_function, unicode_literals
+from pyprofibus.compat import *
+
 import os
 import time
 import errno
 
+__all__ = [
+	"ProfibusError",
+	"bytesToHex",
+	"intToHex",
+	"boolToStr",
+	"fileExists",
+	"monotonic_time",
+	"TimeLimit",
+	"FaultDebouncer",
+]
 
 class ProfibusError(Exception):
-	pass
+	__slots__ = (
+	)
 
-def bytesToHex(b, sep = " "):
+def bytesToHex(b, sep=" "):
+	if b is None:
+		return "None"
+	assert isinstance(b, (bytes, bytearray))
+	if not b:
+		return "Empty"
 	return sep.join("%02X" % c for c in bytearray(b))
 
 def intToHex(val):
 	if val is None:
 		return "None"
+	assert isinstance(val, int)
 	val &= 0xFFFFFFFF
 	if val <= 0xFF:
 		return "0x%02X" % val
@@ -32,11 +52,6 @@ def intToHex(val):
 	else:
 		return "0x%08X" % val
 
-def intListToHex(valList):
-	if valList is None:
-		return "None"
-	return "[%s]" % ", ".join(intToHex(b) for b in valList)
-
 def boolToStr(val):
 	return str(bool(val))
 
@@ -54,7 +69,11 @@ def fileExists(filename):
 	return True
 
 # Monotonic time. Returns a float second count.
-monotonic_time = getattr(time, "monotonic", time.time)
+if isMicropython:
+	def monotonic_time():
+		return time.ticks_ms() / 1e3
+else:
+	monotonic_time = getattr(time, "monotonic", time.time)
 
 class TimeLimit(object):
 	"""Generic timeout helper.
@@ -62,6 +81,12 @@ class TimeLimit(object):
 
 	UNLIMITED	= -1	# No limit
 
+	__slots__ = (
+		"__limit",
+		"__startTime",
+		"__endTime",
+	)
+
 	# limit => The time limit, in seconds.
 	#          Negative value = unlimited.
 	def __init__(self, limit = 0):
@@ -95,6 +120,11 @@ class FaultDebouncer(object):
 	"""Fault counter/debouncer.
 	"""
 
+	__slots__ = (
+		"__countMax",
+		"__count",
+	)
+
 	def __init__(self, countMax = 0xFFFF):
 		self.__countMax = countMax
 		self.reset()
